#include <LiquidCrystal.h>
#include <SPI.h>
#include <nRF24L01.h>
#include <RF24.h>

//nRF pin setup
RF24 radio(8, 9);
const byte address[6] = "00001";

union packet {
  float vals[6];
  byte bytes[4];
} dataPacket;

//LCD pin setup
LiquidCrystal lcd(1, 2, 4, 5, 6, 7);

//LEDS pin setup
int redLed = 10;
int yellowLed = 11;
int greenLed = 12;

//JOYSTICK pin setup
int xAxis = A8;
int yAxis = A9;
int count;

//Potentiometer pin setup
int sensorPin0 = A0;
int sensorPin1 = A1;
int sensorPin2 = A2;
int sensorPin3 = A3;


int arrayStep, arrayMax, Tester, maxStep; // arrayStep-location in the array, arrayMax - maximum number in the array, Tester - switch states, maxStep - help to remap the steps

// Time steps to calculate steps
long previousMillis1 = 0;
long previousMillis2 = 0;
unsigned long currentMillis = millis();
unsigned long currentMicros = micros();

// array to keep sensor values
int SensVal[4];

// arrays to calculate to movment while in play mode
float  dirs[4], stepsCount[4], ist[4], Home[4], steps[4], previousSteps[4], CurrentPos[4];

// arrays to store all of the movments
int joint0[500];
int joint1[500];
int joint2[500];
int joint3[500];

int top = 499;
boolean playmode = false;


void setup() {

  pinMode(22, INPUT); //button

  //setup for nRF
  radio.begin();
  radio.openWritingPipe(address);
  radio.setPALevel(RF24_PA_MIN);
  radio.stopListening();

  //setup for leds
  pinMode(redLed, OUTPUT);
  pinMode(yellowLed, OUTPUT);
  pinMode(greenLed, OUTPUT);

  //setup for lcd
  lcd.begin(16, 2);
  lcd.print("Physical-Twin");
  delay(2000);

}

void loop() {

  currentMillis = millis();
  currentMicros = micros();

  Button(); //Get current state

  if (!playmode) {
    digitalWrite(redLed, HIGH);
    digitalWrite(greenLed, LOW);
    lcd.setCursor(0, 0);
    if (Tester == 1) {
      digitalWrite(yellowLed, HIGH);
      lcd.print("Recording  -  ");
    }
    else {
      digitalWrite(yellowLed, LOW);
      lcd.print("Train - "); lcd.print(arrayMax); lcd.print("/"); lcd.print(top); lcd.print("    ");
    }
    if (currentMillis - previousMillis1 > 25) {
      if (arrayStep < top) {

        previousMillis1 = currentMillis; //reset
        readPot(); //read the minirobot data
        mapping(); //map the values
        record(); //record the values


        lcd.setCursor(0, 1);
        lcd.print((int)ist[0]);
        lcd.print("-");
        lcd.print((int)ist[1]);
        lcd.print("-");
        lcd.print((int)ist[2]);
        lcd.print("-");
        lcd.print((int)ist[3]);
        lcd.print("            ");

        //setting the values to send to the receiver
        dataPacket.vals[0] = analogRead(xAxis);
        dataPacket.vals[1] = analogRead(yAxis);
        dataPacket.vals[2] = ist[0];
        dataPacket.vals[3] = ist[1];
        dataPacket.vals[4] = ist[2];
        dataPacket.vals[5] = ist[3];

        radio.write(&dataPacket.vals, sizeof(dataPacket.vals)); //send movment to receiver
      }
    }
  }

  else if (playmode) {

    digitalWrite(redLed, LOW);
    digitalWrite(yellowLed, LOW);
    digitalWrite(greenLed, HIGH);
    lcd.setCursor(0, 0);

    if (arrayStep < arrayMax) {
      Read();
      play();
      arrayStep++;
      delay(100); //set back to 10 when plugin everything
    }
    else {
      ist[0] = joint0[0], ist[1] = joint1[0], ist[2] = joint2[0], ist[3] = joint3[0];
      play();
      arrayStep = 0;
      delay(1000);
    }

    lcd.print("Play - "); lcd.print(arrayStep); lcd.print("/"); lcd.print(arrayMax); lcd.print("     ");
  }
}

void readPot() {
  SensVal[0] = analogRead(sensorPin0);
  SensVal[1] = analogRead(sensorPin1);
  SensVal[2] = analogRead(sensorPin2);
  SensVal[3] = analogRead(sensorPin3);
}

void mapping() {
  ist[0] = map(SensVal[0], 0, 1023, 0, 180);
  ist[1] = map(SensVal[1], 0, 1023, 0, 180);
  ist[2] = map(SensVal[2], 0, 1023, 0, 180);
  ist[3] = map(SensVal[3], 0, 1023, 0, 180);
}

void record() {
  joint0[arrayStep] = ist[0];
  joint1[arrayStep] = ist[1];
  joint2[arrayStep] = ist[2];
  joint3[arrayStep] = ist[3];
}

void Read() {
  ist[0] = joint0[arrayStep];
  ist[1] = joint1[arrayStep];
  ist[2] = joint2[arrayStep];
  ist[3] = joint3[arrayStep];
}


void Button() {

  if (digitalRead(22) == false) {

    delay(100);

    if (digitalRead(22) == true) { //first click to run record

      if (Tester == 0) {
        Tester = 1;
        previousMillis2 = currentMillis;
      }

      else if ((Tester == 1) && (currentMillis - previousMillis2 < 1500)) //double click to start playmode
      {
        Tester = 2;
      }

      if (playmode) {
        arrayStep = 0;
        arrayMax = 0;
        Tester = 0;
        resetArrays();
        playmode = false;
      }
    }
  }

  if (Tester == 1) { //Record Mode
    arrayStep += 1;
    arrayMax = arrayStep;
    playmode = false;
    lcd.setCursor(11, 0);
    lcd.print(arrayStep);
    delay(100);
  }
  else if (Tester == 2) {
    arrayStep = 0 ;
    previousSteps[0] = joint0[0];
    previousSteps[1] = joint1[0];
    previousSteps[2] = joint2[0];
    previousSteps[3] = joint3[0];
    playmode = true;
  }

  if (currentMillis - previousMillis2 > 2000) // button Status clear
  {
    Tester = 0;
  }
}

void resetArrays() {
  for (int i = 0; i < top; i++) {
    joint0[i] = 0;
    joint1[i] = 0;
    joint2[i] = 0;
    joint3[i] = 0;
  }
}

void play() {
  steps[0] = ist[0] - previousSteps[0];
  steps[1] = ist[1] - previousSteps[1];
  steps[2] = ist[2] - previousSteps[2];
  steps[3] = ist[3] - previousSteps[3];

  maxStep = max(abs(steps[0]), abs(steps[1]));
  maxStep = max(maxStep, abs(steps[2]));
  maxStep = max(maxStep, abs(steps[3]));

  stepsCount[0] = previousSteps[0], stepsCount[1] = previousSteps[1], stepsCount[2] = previousSteps[2], stepsCount[3] = previousSteps[3];



  dirs[0] = steps[0] / maxStep;
  dirs[1] = steps[1] / maxStep;
  dirs[2] = steps[2] / maxStep;
  dirs[3] = steps[3] / maxStep;

  for (int i = 0; i < maxStep - 1; i++) {


    stepsCount[0] = stepsCount[0] + dirs[0];
    stepsCount[1] = stepsCount[1] + dirs[1];
    stepsCount[2] = stepsCount[2] + dirs[2];
    stepsCount[3] = stepsCount[3] + dirs[3];

    lcd.setCursor(0, 1);
    lcd.print((int)stepsCount[0]);
    lcd.print("-");
    lcd.print((int)stepsCount[1]);
    lcd.print("-");
    lcd.print((int)stepsCount[2]);
    lcd.print("-");
    lcd.print((int)stepsCount[3]);
    lcd.print("            ");

    dataPacket.vals[0] = analogRead(xAxis);
    dataPacket.vals[1] = analogRead(yAxis);
    dataPacket.vals[2] = stepsCount[0];
    dataPacket.vals[3] = stepsCount[1];
    dataPacket.vals[4] = stepsCount[2];
    dataPacket.vals[5] = stepsCount[3];

    radio.write(&dataPacket.vals, sizeof(dataPacket.vals));

    delay(35);
  }
  previousSteps[0] = ist[0];
  previousSteps[1] = ist[1];
  previousSteps[2] = ist[2];
  previousSteps[3] = ist[3];
}
